import escapeStringRegexp from 'escape-string-regexp';
import React from 'react';
var stringToRegex = function stringToRegex(str) {
  var caseSens = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  return new RegExp(escapeStringRegexp(str), caseSens ? 'g' : 'gi');
};
var getWordRanges = function getWordRanges(text, word, caseSensitive) {
  var reg = stringToRegex(word, caseSensitive);
  var res = reg.exec(text);
  var ranges = [];
  while (res) {
    var _start = res.index;
    ranges.push([_start, _start + word.length]);
    res = reg.exec(text);
  }
  return ranges;
};

/**
 * cur splitRange is in wordRange, if in range add config
 * @param curRange
 * @param wordRange
 * @returns
 */
var isInRange = function isInRange(curRange, wordRange) {
  return curRange[0] >= wordRange[0] && curRange[1] <= wordRange[1];
};
export var getRangesFromConfig = function getRangesFromConfig(text, configs) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var rangeToConfig = new Map();
  var rangesMap = {};
  var splitIndexes = [];
  configs.forEach(function (config) {
    var word = config.word;
    var ranges = getWordRanges(text, word, options.caseSensitive);
    ranges.forEach(function (range) {
      rangeToConfig.set(range, config);
      splitIndexes = splitIndexes.concat(range);
    });
    rangesMap[word] = ranges;
  });
  splitIndexes.sort(function (a, b) {
    return a - b;
  });
  if (splitIndexes[0] !== 0) {
    splitIndexes = [0].concat(splitIndexes);
  }
  if (splitIndexes[splitIndexes.length - 1] !== text.length) {
    splitIndexes.push(text.length);
  }
  return {
    /** range -> config */
    rangeToConfig: rangeToConfig,
    /** text will split by these ranges */
    splitIndexes: splitIndexes
  };
};
export var toHTML = function toHTML(text, splitIndexes, rangeToConfig, options) {
  var HTag = (options === null || options === void 0 ? void 0 : options.highlightTag) || 'span';
  var NTag = (options === null || options === void 0 ? void 0 : options.normalTag) || React.Fragment;
  var fragments = [];
  var _loop = function _loop() {
    var left = splitIndexes[i];
    var right = splitIndexes[i + 1];
    var configs = [];
    rangeToConfig.forEach(function (config, range) {
      if (isInRange([left, right], range)) {
        configs.push(config);
      }
    });
    var words = text.slice(left, right);
    if (configs.length > 0) {
      var _configs$reduce = configs.reduce(function (res, _ref, index) {
          var className = _ref.className,
            style = _ref.style;
          if (index === 0) {
            res.className = className;
          } else {
            res.className = "".concat(res.className, " ").concat(className);
          }
          res.style = Object.assign(res.style, style);
          return res;
        }, {
          className: '',
          style: {}
        }),
        className = _configs$reduce.className,
        style = _configs$reduce.style;
      fragments.push( /*#__PURE__*/React.createElement(HTag, {
        style: style,
        className: className
      }, words));
    } else {
      fragments.push( /*#__PURE__*/React.createElement(NTag, null, words));
    }
  };
  for (var i = 0; i < splitIndexes.length - 1; i += 1) {
    _loop();
  }
  return fragments;
};

// test
// 'Life, thin and light-off time and time again',
// [
//   {
//     word: 'and',
//     className: 'a',
//     style: {
//       color: 'yellow',
//     },
//   },
//   {
//     word: 'time',
//     className: 'b',
//     style: {
//       color: 'red',
//     }
//   },
//   {
//     word: 'd light-off ti',
//     className: 'c',
//     style: {
//       textDecorator: 'underline',
//     }
//   }
// ]

// rangesMap{
//   "and": [
//       [
//           11,
//           14
//       ],
//       [
//           30,
//           33
//       ]
//   ],
//   "time": [
//       [
//           25,
//           29
//       ],
//       [
//           34,
//           38
//       ]
//   ],
//   "d light-off ti": [
//       [
//           13,
//           27
//       ]
//   ]
// }

// splitIndexes [
//   0,
//   11,
//   13,
//   14,
//   25,
//   27,
//   29,
//   30,
//   33,
//   34,
//   38,
//   44
// ]